@@ -37,6 +37,7 @@ from unittest.mock import patch
 from urllib.parse import unquote
 
 from dulwich import client, file, index, objects, protocol, repo
+from dulwich.porcelain import tag_create
 
 from .. import SkipTest, expectedFailure
 from .utils import (
@@ -226,10 +227,62 @@ class DulwichClientTestBase:
         c = self._client()
         with repo.Repo(os.path.join(self.gitroot, "dest")) as dest:
             result = c.fetch(self._build_path("/server_new.export"), dest)
+            self.assertEqual(
+                {b"HEAD": b"refs/heads/master"},
+                result.symrefs,
+            )
             for r in result.refs.items():
                 dest.refs.set_if_equals(r[0], None, r[1])
             self.assertDestEqualsSrc()
 
+    def test_fetch_pack_with_nondefault_symref(self):
+        c = self._client()
+        src = repo.Repo(os.path.join(self.gitroot, "server_new.export"))
+        src.refs.add_if_new(b"refs/heads/main", src.refs[b"refs/heads/master"])
+        src.refs.set_symbolic_ref(b"HEAD", b"refs/heads/main")
+        with repo.Repo(os.path.join(self.gitroot, "dest")) as dest:
+            result = c.fetch(self._build_path("/server_new.export"), dest)
+            self.assertEqual(
+                {b"HEAD": b"refs/heads/main"},
+                result.symrefs,
+            )
+            for r in result.refs.items():
+                dest.refs.set_if_equals(r[0], None, r[1])
+            self.assertDestEqualsSrc()
+
+    def test_get_refs_with_peeled_tag(self):
+        tag_create(
+            os.path.join(self.gitroot, "server_new.export"),
+            b"v1.0",
+            message="tagging".encode("ascii"),
+            annotated=True,
+        )
+        c = self._client()
+        refs = c.get_refs(self._build_path("/server_new.export"))
+        self.assertEqual(
+            [
+                b"HEAD",
+                b"refs/heads/branch",
+                b"refs/heads/master",
+                b"refs/tags/v1.0",
+                b"refs/tags/v1.0^{}",
+            ],
+            sorted(refs.keys()),
+        )
+
+    def test_get_refs_with_ref_prefix(self):
+        c = self._client()
+        refs = c.get_refs(
+            self._build_path("/server_new.export"), ref_prefix=[b"refs/heads"]
+        )
+        self.assertEqual(
+            [
+                b"refs/heads/branch",
+                b"refs/heads/master",
+            ],
+            sorted(refs.keys()),
+        )
+
     def test_fetch_pack_depth(self):
         c = self._client()
         with repo.Repo(os.path.join(self.gitroot, "dest")) as dest:
